"""add preference for workout chart display

Revision ID: 98688d3d293b
Revises: 4eeeecd3936d
Create Date: 2025-07-19 18:34:49.412724

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '98688d3d293b'
down_revision = '4eeeecd3936d'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table('users', schema=None) as batch_op:
        batch_op.add_column(sa.Column('split_workout_charts', sa.Boolean(), server_default='false', nullable=False))

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table('users', schema=None) as batch_op:
        batch_op.drop_column('split_workout_charts')

    # ### end Alembic commands ###
