"""add equipments visibility

Revision ID: ce68b3914ff7
Revises: 070bd31919c3
Create Date: 2025-02-02 10:29:28.718139

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'ce68b3914ff7'
down_revision = '070bd31919c3'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table('equipments', schema=None) as batch_op:
        batch_op.add_column(sa.Column('visibility', sa.Enum('PUBLIC', 'FOLLOWERS', 'PRIVATE', name='visibility_levels'), server_default='PRIVATE', nullable=False))

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table('equipments', schema=None) as batch_op:
        batch_op.drop_column('visibility')

    # ### end Alembic commands ###
