"""add application config in database

Revision ID: 8a0aad4c838c
Revises: 1345afe3b11d
Create Date: 2019-11-13 13:14:20.147296

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '8a0aad4c838c'
down_revision = '1345afe3b11d'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('app_config',
    sa.Column('id', sa.Integer(), autoincrement=True, nullable=False),
    sa.Column('max_users', sa.Integer(), nullable=False),
    sa.Column('gpx_limit_import', sa.Integer(), nullable=False),
    sa.Column('max_single_file_size', sa.Integer(), nullable=False),
    sa.Column('max_zip_file_size', sa.Integer(), nullable=False),
    sa.PrimaryKeyConstraint('id')
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('app_config')
    # ### end Alembic commands ###
